@@ -5,7 +5,8 @@
 
 .. cmdoption:: -c, --concurrency
 
-    Number of child processes processing the queue.
+    Number of child processes processing the queue. The default
+    is the number of CPUs available on your system.
 
 .. cmdoption:: -f, --logfile
 
@@ -20,10 +21,19 @@
 
     Path to pidfile.
 
+.. cmdoption:: -s, --statistics
+
+    Turn on reporting of statistics (remember to flush the statistics message
+    queue from time to time).
+
 .. cmdoption:: -d, --detach, --daemon
 
     Run in the background as a daemon.
 
+.. cmdoption:: -S, --supervised
+
+    Restart the worker server if it dies.
+
 .. cmdoption:: --discard
 
     Discard all waiting tasks before the daemon is started.
@@ -53,12 +63,19 @@
 """
 import os
 import sys
+CAN_DETACH = True
+try:
+    import resource
+except ImportError:
+    CAN_DETACH = False
 sys.path.append(os.getcwd())
 django_project_dir = os.environ.get("DJANGO_PROJECT_DIR")
 if django_project_dir:
     sys.path.append(django_project_dir)
 
 from django.conf import settings
+from celery import __version__
+from celery.supervisor import OFASupervisor
 from celery.log import emergency_error
 from celery.conf import LOG_LEVELS, DAEMON_LOG_FILE, DAEMON_LOG_LEVEL
 from celery.conf import DAEMON_CONCURRENCY, DAEMON_PID_FILE
@@ -67,20 +84,24 @@ from celery import conf
 from celery import discovery
 from celery.task import discard_all
 from celery.worker import WorkController
+from carrot.connection import DjangoAMQPConnection
+from celery.messaging import TaskConsumer, StatsConsumer
 import multiprocessing
 import traceback
 import optparse
 import atexit
-from daemon import DaemonContext
-from daemon.pidlockfile import PIDLockFile
-import errno
+
+USE_STATISTICS = getattr(settings, "CELERY_STATISTICS", False)
+# Make sure the setting exists.
+settings.CELERY_STATISTICS = USE_STATISTICS
 
 STARTUP_INFO_FMT = """
-    * Celery loading with the following configuration
-        * Broker -> amqp://%(vhost)s@%(host)s:%(port)s
-        * Exchange -> %(exchange)s (%(exchange_type)s)
-        * Consumer -> Queue:%(consumer_queue)s Routing:%(consumer_rkey)s
-        * Concurrency:%(concurrency)s
+Configuration ->
+    * Broker -> amqp://%(vhost)s@%(host)s:%(port)s
+    * Exchange -> %(exchange)s (%(exchange_type)s)
+    * Consumer -> Queue:%(consumer_queue)s Routing:%(consumer_rkey)s
+    * Concurrency -> %(concurrency)s
+    * Statistics -> %(statistics)s
 """.strip()
 
 OPTION_LIST = (
@@ -92,6 +113,9 @@ OPTION_LIST = (
             help="Discard all waiting tasks before the server is started. "
                  "WARNING: This is unrecoverable, and the tasks will be "
                  "deleted from the messaging server."),
+    optparse.make_option('-s', '--statistics', default=USE_STATISTICS,
+            action="store_true", dest="statistics",
+            help="Collect statistics."),
     optparse.make_option('-f', '--logfile', default=DAEMON_LOG_FILE,
             action="store", dest="logfile",
             help="Path to log file."),
@@ -104,6 +128,9 @@ OPTION_LIST = (
     optparse.make_option('-d', '--detach', '--daemon', default=False,
             action="store_true", dest="detach",
             help="Run in the background as a daemon."),
+    optparse.make_option('-S', '--supervised', default=False,
+            action="store_true", dest="supervised",
+            help="Restart the worker server if it dies."),
     optparse.make_option('-u', '--uid', default=None,
             action="store", dest="uid",
             help="User-id to run celeryd as when in daemon mode."),
@@ -133,6 +160,8 @@ def acquire_pidlock(pidfile):
     running in the background somewhere.
 
     """
+    from daemon.pidlockfile import PIDLockFile
+    import errno
     pidlock = PIDLockFile(pidfile)
     if not pidlock.is_locked():
         return pidlock
@@ -142,7 +171,7 @@ def acquire_pidlock(pidfile):
     except os.error, exc:
         if exc.errno == errno.ESRCH:
             sys.stderr.write("Stale pidfile exists. Removing it.\n")
-            pidlock.release()
+            os.unlink(pidfile)
             return PIDLockFile(pidfile)
     else:
         raise SystemExit(
@@ -155,11 +184,18 @@ def acquire_pidlock(pidfile):
 def run_worker(concurrency=DAEMON_CONCURRENCY, detach=False,
         loglevel=DAEMON_LOG_LEVEL, logfile=DAEMON_LOG_FILE, discard=False,
         pidfile=DAEMON_PID_FILE, umask=0, uid=None, gid=None,
-        working_directory=None, chroot=None, **kwargs):
+        supervised=False, working_directory=None, chroot=None,
+        statistics=None, **kwargs):
     """Starts the celery worker server."""
 
+    print("Celery %s is starting." % __version__)
+
+    if statistics:
+        settings.CELERY_STATISTICS = statistics
+
     if not concurrency:
         concurrency = multiprocessing.cpu_count()
+
     if settings.DATABASE_ENGINE == "sqlite3" and concurrency > 1:
         import warnings
         warnings.warn("The sqlite3 database engine doesn't support "
@@ -193,9 +229,15 @@ def run_worker(concurrency=DAEMON_CONCURRENCY, detach=False,
             "concurrency": concurrency,
             "loglevel": loglevel,
             "pidfile": pidfile,
+            "statistics": settings.CELERY_STATISTICS and "ON" or "OFF",
     })
 
+    print("Celery has started.")
     if detach:
+        if not CAN_DETACH:
+            raise RuntimeError(
+                    "This operating system doesn't support detach. ")
+        from daemon import DaemonContext
         # Since without stderr any errors will be silently suppressed,
         # we need to know that we have access to the logfile
         if logfile:
@@ -206,7 +248,6 @@ def run_worker(concurrency=DAEMON_CONCURRENCY, detach=False,
         uid = uid and int(uid) or os.geteuid()
         gid = gid and int(gid) or os.getegid()
         working_directory = working_directory or os.getcwd()
-        print("* Launching celeryd in the background...")
         context = DaemonContext(chroot_directory=chroot,
                                 working_directory=working_directory,
                                 umask=umask,
@@ -216,17 +257,25 @@ def run_worker(concurrency=DAEMON_CONCURRENCY, detach=False,
         context.open()
 
     discovery.autodiscover()
-    worker = WorkController(concurrency=concurrency,
-                            loglevel=loglevel,
-                            logfile=logfile,
-                            is_detached=detach)
-    try:
-        worker.run()
-    except Exception, e:
-        emergency_error(logfile, "celeryd raised exception %s: %s\n%s" % (
+
+    def run_worker():
+        worker = WorkController(concurrency=concurrency,
+                                loglevel=loglevel,
+                                logfile=logfile,
+                                is_detached=detach)
+        try:
+            worker.start()
+        except Exception, e:
+            emergency_error(logfile, "celeryd raised exception %s: %s\n%s" % (
                             e.__class__, e, traceback.format_exc()))
+
+    try:
+        if supervised:
+            OFASupervisor(target=run_worker).start()
+        else:
+            run_worker()
     except:
-        if daemon:
+        if detach:
             context.close()
         raise
 
