@@ -18,11 +18,13 @@ class InfinityThread(threading.Thread):
     the :meth:`stop` method.
 
     """
+    is_infinite = True
 
     def __init__(self):
         super(InfinityThread, self).__init__()
         self._shutdown = threading.Event()
         self._stopped = threading.Event()
+        self.setDaemon(True)
 
     def run(self):
         """This is the body of the thread.
@@ -30,7 +32,7 @@ class InfinityThread(threading.Thread):
         To start the thread use :meth:`start` instead.
 
         """
-        while True:
+        while self.is_infinite:
             if self._shutdown.isSet():
                 break
             self.on_iteration()
