@@ -3,7 +3,6 @@
 Jobs Executable by the Worker Server.
 
 """
-from celery.conf import SEND_CELERY_TASK_ERROR_EMAILS
 from celery.registry import tasks, NotRegistered
 from celery.datastructures import ExceptionInfo
 from celery.backends import default_backend
@@ -183,8 +182,8 @@ class TaskWrapper(object):
         kwargs = message_data["kwargs"]
 
         # Convert any unicode keys in the keyword arguments to ascii.
-        kwargs = dict([(key.encode("utf-8"), value)
-                    for key, value in kwargs.items()])
+        kwargs = dict((key.encode("utf-8"), value)
+                        for key, value in kwargs.items())
 
         if task_name not in tasks:
             raise NotRegistered(task_name)
@@ -233,6 +232,8 @@ class TaskWrapper(object):
 
     def on_failure(self, exc_info, meta):
         """The handler used if the task raised an exception."""
+        from celery.conf import SEND_CELERY_TASK_ERROR_EMAILS
+
         task_id = meta.get("task_id")
         task_name = meta.get("task_name")
         context = {
